@@ -2,20 +2,30 @@ import { DataFrame, Field } from './dataFrame';
 import { RegistryItemWithOptions } from '../utils/Registry';
 
 /**
- * Immutable data transformation
+ * Function that transform data frames (AKA transformer)
  */
 export type DataTransformer = (data: DataFrame[]) => DataFrame[];
 
 export interface DataTransformerInfo<TOptions = any> extends RegistryItemWithOptions {
+  /**
+   * Function that configures transformation and returns a transformer
+   * @param options
+   */
   transformer: (options: TOptions) => DataTransformer;
 }
 
 export interface DataTransformerConfig<TOptions = any> {
+  /**
+   * Unique identifier of transformer
+   */
   id: string;
+  /**
+   * Options to be passed to the transformer
+   */
   options: TOptions;
 }
 
-export type FieldMatcher = (field: Field) => boolean;
+export type FieldMatcher = (field: Field, frame: DataFrame, allFrames: DataFrame[]) => boolean;
 export type FrameMatcher = (frame: DataFrame) => boolean;
 
 export interface FieldMatcherInfo<TOptions = any> extends RegistryItemWithOptions<TOptions> {
