@@ -1,4 +1,5 @@
-import { LogLevel } from '../types/logs';
+import { LogLevel, LogsModel, LogRowModel, LogsSortOrder } from '../types/logs';
+import { MutableDataFrame } from '../dataframe/MutableDataFrame';
 import {
   getLogLevel,
   calculateLogsLabelStats,
@@ -6,6 +7,8 @@ import {
   getParser,
   LogsParsers,
   calculateStats,
+  getLogLevelFromKey,
+  sortLogsResult,
 } from './logs';
 
 describe('getLoglevel()', () => {
@@ -14,7 +17,7 @@ describe('getLoglevel()', () => {
   });
 
   it('returns no log level on when level is part of a word', () => {
-    expect(getLogLevel('this is information')).toBe(LogLevel.unknown);
+    expect(getLogLevel('who warns us')).toBe(LogLevel.unknown);
   });
 
   it('returns same log level for long and short version', () => {
@@ -23,6 +26,10 @@ describe('getLoglevel()', () => {
     expect(getLogLevel('[Warn]')).toBe('warning');
   });
 
+  it('returns correct log level when level is capitalized', () => {
+    expect(getLogLevel('WARN')).toBe(LogLevel.warn);
+  });
+
   it('returns log level on line contains a log level', () => {
     expect(getLogLevel('warn: it is looking bad')).toBe(LogLevel.warn);
     expect(getLogLevel('2007-12-12 12:12:12 [WARN]: it is looking bad')).toBe(LogLevel.warn);
@@ -30,6 +37,19 @@ describe('getLoglevel()', () => {
 
   it('returns first log level found', () => {
     expect(getLogLevel('WARN this could be a debug message')).toBe(LogLevel.warn);
+    expect(getLogLevel('WARN this is a non-critical message')).toBe(LogLevel.warn);
+  });
+});
+
+describe('getLogLevelFromKey()', () => {
+  it('returns correct log level', () => {
+    expect(getLogLevelFromKey('info')).toBe(LogLevel.info);
+  });
+  it('returns correct log level when level is capitalized', () => {
+    expect(getLogLevelFromKey('INFO')).toBe(LogLevel.info);
+  });
+  it('returns unknown log level when level is integer', () => {
+    expect(getLogLevelFromKey(1)).toBe(LogLevel.unknown);
   });
 });
 
@@ -98,21 +118,25 @@ describe('LogsParsers', () => {
     test('should return parsed fields', () => {
       expect(
         parser.getFields(
-          'foo=bar baz="42 + 1" msg="[resolver] received A record \\"127.0.0.1\\" for \\"localhost.\\" from udp:192.168.65.1"'
+          'foo=bar baz="42 + 1" msg="[resolver] received A record \\"127.0.0.1\\" for \\"localhost.\\" from udp:192.168.65.1" time(ms)=50 label{foo}=bar'
         )
       ).toEqual([
         'foo=bar',
         'baz="42 + 1"',
         'msg="[resolver] received A record \\"127.0.0.1\\" for \\"localhost.\\" from udp:192.168.65.1"',
+        'time(ms)=50',
+        'label{foo}=bar',
       ]);
     });
 
     test('should return label for field', () => {
       expect(parser.getLabelFromField('foo=bar')).toBe('foo');
+      expect(parser.getLabelFromField('time(ms)=50')).toBe('time(ms)');
     });
 
     test('should return value for field', () => {
       expect(parser.getValueFromField('foo=bar')).toBe('bar');
+      expect(parser.getValueFromField('time(ms)=50')).toBe('50');
       expect(
         parser.getValueFromField(
           'msg="[resolver] received A record \\"127.0.0.1\\" for \\"localhost.\\" from udp:192.168.65.1"'
@@ -126,6 +150,13 @@ describe('LogsParsers', () => {
       expect(match).toBeDefined();
       expect(match![1]).toBe('bar');
     });
+
+    test('should build a valid complex value matcher', () => {
+      const matcher = parser.buildMatcher('time(ms)');
+      const match = 'time(ms)=50'.match(matcher);
+      expect(match).toBeDefined();
+      expect(match![1]).toBe('50');
+    });
   });
 
   describe('JSON', () => {
@@ -255,3 +286,69 @@ describe('getParser()', () => {
     expect(getParser('{"foo": "bar", "baz": "41 + 1"}')).toEqual(LogsParsers.JSON);
   });
 });
+
+describe('sortLogsResult', () => {
+  const firstRow: LogRowModel = {
+    rowIndex: 0,
+    entryFieldIndex: 0,
+    dataFrame: new MutableDataFrame(),
+    entry: '',
+    hasAnsi: false,
+    labels: {},
+    logLevel: LogLevel.info,
+    raw: '',
+    timeEpochMs: 0,
+    timeEpochNs: '0',
+    timeFromNow: '',
+    timeLocal: '',
+    timeUtc: '',
+    uid: '1',
+  };
+  const sameAsFirstRow = firstRow;
+  const secondRow: LogRowModel = {
+    rowIndex: 1,
+    entryFieldIndex: 0,
+    dataFrame: new MutableDataFrame(),
+    entry: '',
+    hasAnsi: false,
+    labels: {},
+    logLevel: LogLevel.info,
+    raw: '',
+    timeEpochMs: 10,
+    timeEpochNs: '10000000',
+    timeFromNow: '',
+    timeLocal: '',
+    timeUtc: '',
+    uid: '2',
+  };
+
+  describe('when called with LogsSortOrder.Descending', () => {
+    it('then it should sort descending', () => {
+      const logsResult: LogsModel = {
+        rows: [firstRow, sameAsFirstRow, secondRow],
+        hasUniqueLabels: false,
+      };
+      const result = sortLogsResult(logsResult, LogsSortOrder.Descending);
+
+      expect(result).toEqual({
+        rows: [secondRow, firstRow, sameAsFirstRow],
+        hasUniqueLabels: false,
+      });
+    });
+  });
+
+  describe('when called with LogsSortOrder.Ascending', () => {
+    it('then it should sort ascending', () => {
+      const logsResult: LogsModel = {
+        rows: [secondRow, firstRow, sameAsFirstRow],
+        hasUniqueLabels: false,
+      };
+      const result = sortLogsResult(logsResult, LogsSortOrder.Ascending);
+
+      expect(result).toEqual({
+        rows: [firstRow, sameAsFirstRow, secondRow],
+        hasUniqueLabels: false,
+      });
+    });
+  });
+});
