@@ -9,7 +9,8 @@ import (
 	"time"
 
 	"github.com/grafana/grafana/pkg/bus"
-	m "github.com/grafana/grafana/pkg/models"
+	"github.com/grafana/grafana/pkg/components/securejsondata"
+	"github.com/grafana/grafana/pkg/models"
 	"github.com/grafana/grafana/pkg/util"
 )
 
@@ -29,12 +30,21 @@ func init() {
 	bus.AddHandler("sql", GetAlertNotificationsWithUidToSend)
 }
 
-func DeleteAlertNotification(cmd *m.DeleteAlertNotificationCommand) error {
+func DeleteAlertNotification(cmd *models.DeleteAlertNotificationCommand) error {
 	return inTransaction(func(sess *DBSession) error {
 		sql := "DELETE FROM alert_notification WHERE alert_notification.org_id = ? AND alert_notification.id = ?"
-		if _, err := sess.Exec(sql, cmd.OrgId, cmd.Id); err != nil {
+		res, err := sess.Exec(sql, cmd.OrgId, cmd.Id)
+		if err != nil {
 			return err
 		}
+		rowsAffected, err := res.RowsAffected()
+		if err != nil {
+			return err
+		}
+
+		if rowsAffected == 0 {
+			return models.ErrAlertNotificationNotFound
+		}
 
 		if _, err := sess.Exec("DELETE FROM alert_notification_state WHERE alert_notification_state.org_id = ? AND alert_notification_state.notifier_id = ?", cmd.OrgId, cmd.Id); err != nil {
 			return err
@@ -44,35 +54,64 @@ func DeleteAlertNotification(cmd *m.DeleteAlertNotificationCommand) error {
 	})
 }
 
-func DeleteAlertNotificationWithUid(cmd *m.DeleteAlertNotificationWithUidCommand) error {
-	existingNotification := &m.GetAlertNotificationsWithUidQuery{OrgId: cmd.OrgId, Uid: cmd.Uid}
+func DeleteAlertNotificationWithUid(cmd *models.DeleteAlertNotificationWithUidCommand) error {
+	existingNotification := &models.GetAlertNotificationsWithUidQuery{OrgId: cmd.OrgId, Uid: cmd.Uid}
 	if err := getAlertNotificationWithUidInternal(existingNotification, newSession()); err != nil {
 		return err
 	}
 
-	if existingNotification.Result != nil {
-		deleteCommand := &m.DeleteAlertNotificationCommand{
-			Id:    existingNotification.Result.Id,
-			OrgId: existingNotification.Result.OrgId,
-		}
-		if err := bus.Dispatch(deleteCommand); err != nil {
-			return err
-		}
+	if existingNotification.Result == nil {
+		return models.ErrAlertNotificationNotFound
+	}
+
+	cmd.DeletedAlertNotificationId = existingNotification.Result.Id
+	deleteCommand := &models.DeleteAlertNotificationCommand{
+		Id:    existingNotification.Result.Id,
+		OrgId: existingNotification.Result.OrgId,
+	}
+	if err := bus.Dispatch(deleteCommand); err != nil {
+		return err
 	}
 
 	return nil
 }
 
-func GetAlertNotifications(query *m.GetAlertNotificationsQuery) error {
+func GetAlertNotifications(query *models.GetAlertNotificationsQuery) error {
 	return getAlertNotificationInternal(query, newSession())
 }
 
-func GetAlertNotificationsWithUid(query *m.GetAlertNotificationsWithUidQuery) error {
+func (ss *SqlStore) addAlertNotificationUidByIdHandler() {
+	bus.AddHandler("sql", ss.GetAlertNotificationUidWithId)
+}
+
+func (ss *SqlStore) GetAlertNotificationUidWithId(query *models.GetAlertNotificationUidQuery) error {
+	cacheKey := newAlertNotificationUidCacheKey(query.OrgId, query.Id)
+
+	if cached, found := ss.CacheService.Get(cacheKey); found {
+		query.Result = cached.(string)
+		return nil
+	}
+
+	err := getAlertNotificationUidInternal(query, newSession())
+	if err != nil {
+		return err
+	}
+
+	ss.CacheService.Set(cacheKey, query.Result, -1) //Infinite, never changes
+
+	return nil
+}
+
+func newAlertNotificationUidCacheKey(orgID, notificationId int64) string {
+	return fmt.Sprintf("notification-uid-by-org-%d-and-id-%d", orgID, notificationId)
+}
+
+func GetAlertNotificationsWithUid(query *models.GetAlertNotificationsWithUidQuery) error {
 	return getAlertNotificationWithUidInternal(query, newSession())
 }
 
-func GetAllAlertNotifications(query *m.GetAllAlertNotificationsQuery) error {
-	results := make([]*m.AlertNotification, 0)
+func GetAllAlertNotifications(query *models.GetAllAlertNotificationsQuery) error {
+	results := make([]*models.AlertNotification, 0)
 	if err := x.Where("org_id = ?", query.OrgId).Find(&results); err != nil {
 		return err
 	}
@@ -81,7 +120,7 @@ func GetAllAlertNotifications(query *m.GetAllAlertNotificationsQuery) error {
 	return nil
 }
 
-func GetAlertNotificationsWithUidToSend(query *m.GetAlertNotificationsWithUidToSendQuery) error {
+func GetAlertNotificationsWithUidToSend(query *models.GetAlertNotificationsWithUidToSendQuery) error {
 	var sql bytes.Buffer
 	params := make([]interface{}, 0)
 
@@ -94,6 +133,7 @@ func GetAlertNotificationsWithUidToSend(query *m.GetAlertNotificationsWithUidToS
 										alert_notification.created,
 										alert_notification.updated,
 										alert_notification.settings,
+										alert_notification.secure_settings,
 										alert_notification.is_default,
 										alert_notification.disable_resolve_message,
 										alert_notification.send_reminder,
@@ -115,7 +155,7 @@ func GetAlertNotificationsWithUidToSend(query *m.GetAlertNotificationsWithUidToS
 	}
 	sql.WriteString(`)`)
 
-	results := make([]*m.AlertNotification, 0)
+	results := make([]*models.AlertNotification, 0)
 	if err := x.SQL(sql.String(), params...).Find(&results); err != nil {
 		return err
 	}
@@ -124,7 +164,36 @@ func GetAlertNotificationsWithUidToSend(query *m.GetAlertNotificationsWithUidToS
 	return nil
 }
 
-func getAlertNotificationInternal(query *m.GetAlertNotificationsQuery, sess *DBSession) error {
+func getAlertNotificationUidInternal(query *models.GetAlertNotificationUidQuery, sess *DBSession) error {
+	var sql bytes.Buffer
+	params := make([]interface{}, 0)
+
+	sql.WriteString(`SELECT
+										alert_notification.uid
+										FROM alert_notification
+	  							`)
+
+	sql.WriteString(` WHERE alert_notification.org_id = ?`)
+	params = append(params, query.OrgId)
+
+	sql.WriteString(` AND alert_notification.id = ?`)
+	params = append(params, query.Id)
+
+	results := make([]string, 0)
+	if err := sess.SQL(sql.String(), params...).Find(&results); err != nil {
+		return err
+	}
+
+	if len(results) == 0 {
+		return fmt.Errorf("Alert notification [ Id: %v, OrgId: %v ] not found", query.Id, query.OrgId)
+	}
+
+	query.Result = results[0]
+
+	return nil
+}
+
+func getAlertNotificationInternal(query *models.GetAlertNotificationsQuery, sess *DBSession) error {
 	var sql bytes.Buffer
 	params := make([]interface{}, 0)
 
@@ -137,6 +206,7 @@ func getAlertNotificationInternal(query *m.GetAlertNotificationsQuery, sess *DBS
 										alert_notification.created,
 										alert_notification.updated,
 										alert_notification.settings,
+										alert_notification.secure_settings,
 										alert_notification.is_default,
 										alert_notification.disable_resolve_message,
 										alert_notification.send_reminder,
@@ -159,7 +229,7 @@ func getAlertNotificationInternal(query *m.GetAlertNotificationsQuery, sess *DBS
 		}
 	}
 
-	results := make([]*m.AlertNotification, 0)
+	results := make([]*models.AlertNotification, 0)
 	if err := sess.SQL(sql.String(), params...).Find(&results); err != nil {
 		return err
 	}
@@ -173,7 +243,7 @@ func getAlertNotificationInternal(query *m.GetAlertNotificationsQuery, sess *DBS
 	return nil
 }
 
-func getAlertNotificationWithUidInternal(query *m.GetAlertNotificationsWithUidQuery, sess *DBSession) error {
+func getAlertNotificationWithUidInternal(query *models.GetAlertNotificationsWithUidQuery, sess *DBSession) error {
 	var sql bytes.Buffer
 	params := make([]interface{}, 0)
 
@@ -186,6 +256,7 @@ func getAlertNotificationWithUidInternal(query *m.GetAlertNotificationsWithUidQu
 										alert_notification.created,
 										alert_notification.updated,
 										alert_notification.settings,
+										alert_notification.secure_settings,
 										alert_notification.is_default,
 										alert_notification.disable_resolve_message,
 										alert_notification.send_reminder,
@@ -196,7 +267,7 @@ func getAlertNotificationWithUidInternal(query *m.GetAlertNotificationsWithUidQu
 	sql.WriteString(` WHERE alert_notification.org_id = ? AND alert_notification.uid = ?`)
 	params = append(params, query.OrgId, query.Uid)
 
-	results := make([]*m.AlertNotification, 0)
+	results := make([]*models.AlertNotification, 0)
 	if err := sess.SQL(sql.String(), params...).Find(&results); err != nil {
 		return err
 	}
@@ -210,7 +281,7 @@ func getAlertNotificationWithUidInternal(query *m.GetAlertNotificationsWithUidQu
 	return nil
 }
 
-func CreateAlertNotificationCommand(cmd *m.CreateAlertNotificationCommand) error {
+func CreateAlertNotificationCommand(cmd *models.CreateAlertNotificationCommand) error {
 	return inTransaction(func(sess *DBSession) error {
 		if cmd.Uid == "" {
 			uid, uidGenerationErr := generateNewAlertNotificationUid(sess, cmd.OrgId)
@@ -220,7 +291,7 @@ func CreateAlertNotificationCommand(cmd *m.CreateAlertNotificationCommand) error
 
 			cmd.Uid = uid
 		}
-		existingQuery := &m.GetAlertNotificationsWithUidQuery{OrgId: cmd.OrgId, Uid: cmd.Uid}
+		existingQuery := &models.GetAlertNotificationsWithUidQuery{OrgId: cmd.OrgId, Uid: cmd.Uid}
 		err := getAlertNotificationWithUidInternal(existingQuery, sess)
 
 		if err != nil {
@@ -232,7 +303,7 @@ func CreateAlertNotificationCommand(cmd *m.CreateAlertNotificationCommand) error
 		}
 
 		// check if name exists
-		sameNameQuery := &m.GetAlertNotificationsQuery{OrgId: cmd.OrgId, Name: cmd.Name}
+		sameNameQuery := &models.GetAlertNotificationsQuery{OrgId: cmd.OrgId, Name: cmd.Name}
 		if err := getAlertNotificationInternal(sameNameQuery, sess); err != nil {
 			return err
 		}
@@ -244,7 +315,7 @@ func CreateAlertNotificationCommand(cmd *m.CreateAlertNotificationCommand) error
 		var frequency time.Duration
 		if cmd.SendReminder {
 			if cmd.Frequency == "" {
-				return m.ErrNotificationFrequencyNotFound
+				return models.ErrNotificationFrequencyNotFound
 			}
 
 			frequency, err = time.ParseDuration(cmd.Frequency)
@@ -253,12 +324,20 @@ func CreateAlertNotificationCommand(cmd *m.CreateAlertNotificationCommand) error
 			}
 		}
 
-		alertNotification := &m.AlertNotification{
+		// delete empty keys
+		for k, v := range cmd.SecureSettings {
+			if v == "" {
+				delete(cmd.SecureSettings, k)
+			}
+		}
+
+		alertNotification := &models.AlertNotification{
 			Uid:                   cmd.Uid,
 			OrgId:                 cmd.OrgId,
 			Name:                  cmd.Name,
 			Type:                  cmd.Type,
 			Settings:              cmd.Settings,
+			SecureSettings:        securejsondata.GetEncryptedJsonData(cmd.SecureSettings),
 			SendReminder:          cmd.SendReminder,
 			DisableResolveMessage: cmd.DisableResolveMessage,
 			Frequency:             frequency,
@@ -279,7 +358,7 @@ func CreateAlertNotificationCommand(cmd *m.CreateAlertNotificationCommand) error
 func generateNewAlertNotificationUid(sess *DBSession, orgId int64) (string, error) {
 	for i := 0; i < 3; i++ {
 		uid := util.GenerateShortUID()
-		exists, err := sess.Where("org_id=? AND uid=?", orgId, uid).Get(&m.AlertNotification{})
+		exists, err := sess.Where("org_id=? AND uid=?", orgId, uid).Get(&models.AlertNotification{})
 		if err != nil {
 			return "", err
 		}
@@ -289,19 +368,23 @@ func generateNewAlertNotificationUid(sess *DBSession, orgId int64) (string, erro
 		}
 	}
 
-	return "", m.ErrAlertNotificationFailedGenerateUniqueUid
+	return "", models.ErrAlertNotificationFailedGenerateUniqueUid
 }
 
-func UpdateAlertNotification(cmd *m.UpdateAlertNotificationCommand) error {
+func UpdateAlertNotification(cmd *models.UpdateAlertNotificationCommand) error {
 	return inTransaction(func(sess *DBSession) (err error) {
-		current := m.AlertNotification{}
+		current := models.AlertNotification{}
 
 		if _, err = sess.ID(cmd.Id).Get(&current); err != nil {
 			return err
 		}
 
+		if current.Id == 0 {
+			return models.ErrAlertNotificationNotFound
+		}
+
 		// check if name exists
-		sameNameQuery := &m.GetAlertNotificationsQuery{OrgId: cmd.OrgId, Name: cmd.Name}
+		sameNameQuery := &models.GetAlertNotificationsQuery{OrgId: cmd.OrgId, Name: cmd.Name}
 		if err := getAlertNotificationInternal(sameNameQuery, sess); err != nil {
 			return err
 		}
@@ -310,8 +393,16 @@ func UpdateAlertNotification(cmd *m.UpdateAlertNotificationCommand) error {
 			return fmt.Errorf("Alert notification name %s already exists", cmd.Name)
 		}
 
+		// delete empty keys
+		for k, v := range cmd.SecureSettings {
+			if v == "" {
+				delete(cmd.SecureSettings, k)
+			}
+		}
+
 		current.Updated = time.Now()
 		current.Settings = cmd.Settings
+		current.SecureSettings = securejsondata.GetEncryptedJsonData(cmd.SecureSettings)
 		current.Name = cmd.Name
 		current.Type = cmd.Type
 		current.IsDefault = cmd.IsDefault
@@ -324,7 +415,7 @@ func UpdateAlertNotification(cmd *m.UpdateAlertNotificationCommand) error {
 
 		if current.SendReminder {
 			if cmd.Frequency == "" {
-				return m.ErrNotificationFrequencyNotFound
+				return models.ErrNotificationFrequencyNotFound
 			}
 
 			frequency, err := time.ParseDuration(cmd.Frequency)
@@ -348,8 +439,8 @@ func UpdateAlertNotification(cmd *m.UpdateAlertNotificationCommand) error {
 	})
 }
 
-func UpdateAlertNotificationWithUid(cmd *m.UpdateAlertNotificationWithUidCommand) error {
-	getAlertNotificationWithUidQuery := &m.GetAlertNotificationsWithUidQuery{OrgId: cmd.OrgId, Uid: cmd.Uid}
+func UpdateAlertNotificationWithUid(cmd *models.UpdateAlertNotificationWithUidCommand) error {
+	getAlertNotificationWithUidQuery := &models.GetAlertNotificationsWithUidQuery{OrgId: cmd.OrgId, Uid: cmd.Uid}
 
 	if err := getAlertNotificationWithUidInternal(getAlertNotificationWithUidQuery, newSession()); err != nil {
 		return err
@@ -358,14 +449,14 @@ func UpdateAlertNotificationWithUid(cmd *m.UpdateAlertNotificationWithUidCommand
 	current := getAlertNotificationWithUidQuery.Result
 
 	if current == nil {
-		return fmt.Errorf("Cannot update, alert notification uid %s doesn't exist", cmd.Uid)
+		return models.ErrAlertNotificationNotFound
 	}
 
 	if cmd.NewUid == "" {
 		cmd.NewUid = cmd.Uid
 	}
 
-	updateNotification := &m.UpdateAlertNotificationCommand{
+	updateNotification := &models.UpdateAlertNotificationCommand{
 		Id:                    current.Id,
 		Uid:                   cmd.NewUid,
 		Name:                  cmd.Name,
@@ -375,6 +466,7 @@ func UpdateAlertNotificationWithUid(cmd *m.UpdateAlertNotificationWithUidCommand
 		Frequency:             cmd.Frequency,
 		IsDefault:             cmd.IsDefault,
 		Settings:              cmd.Settings,
+		SecureSettings:        cmd.SecureSettings,
 
 		OrgId: cmd.OrgId,
 	}
@@ -388,10 +480,10 @@ func UpdateAlertNotificationWithUid(cmd *m.UpdateAlertNotificationWithUidCommand
 	return nil
 }
 
-func SetAlertNotificationStateToCompleteCommand(ctx context.Context, cmd *m.SetAlertNotificationStateToCompleteCommand) error {
+func SetAlertNotificationStateToCompleteCommand(ctx context.Context, cmd *models.SetAlertNotificationStateToCompleteCommand) error {
 	return inTransactionCtx(ctx, func(sess *DBSession) error {
 		version := cmd.Version
-		var current m.AlertNotificationState
+		var current models.AlertNotificationState
 		if _, err := sess.ID(cmd.Id).Get(&current); err != nil {
 			return err
 		}
@@ -404,7 +496,7 @@ func SetAlertNotificationStateToCompleteCommand(ctx context.Context, cmd *m.SetA
 		WHERE
 			id = ?`
 
-		_, err := sess.Exec(sql, m.AlertNotificationStateCompleted, newVersion, timeNow().Unix(), cmd.Id)
+		_, err := sess.Exec(sql, models.AlertNotificationStateCompleted, newVersion, timeNow().Unix(), cmd.Id)
 		if err != nil {
 			return err
 		}
@@ -417,7 +509,7 @@ func SetAlertNotificationStateToCompleteCommand(ctx context.Context, cmd *m.SetA
 	})
 }
 
-func SetAlertNotificationStateToPendingCommand(ctx context.Context, cmd *m.SetAlertNotificationStateToPendingCommand) error {
+func SetAlertNotificationStateToPendingCommand(ctx context.Context, cmd *models.SetAlertNotificationStateToPendingCommand) error {
 	return withDbSession(ctx, func(sess *DBSession) error {
 		newVersion := cmd.Version + 1
 		sql := `UPDATE alert_notification_state SET
@@ -430,7 +522,7 @@ func SetAlertNotificationStateToPendingCommand(ctx context.Context, cmd *m.SetAl
 			(version = ? OR alert_rule_state_updated_version < ?)`
 
 		res, err := sess.Exec(sql,
-			m.AlertNotificationStatePending,
+			models.AlertNotificationStatePending,
 			newVersion,
 			timeNow().Unix(),
 			cmd.AlertRuleStateUpdatedVersion,
@@ -444,7 +536,7 @@ func SetAlertNotificationStateToPendingCommand(ctx context.Context, cmd *m.SetAl
 
 		affected, _ := res.RowsAffected()
 		if affected == 0 {
-			return m.ErrAlertNotificationStateVersionConflict
+			return models.ErrAlertNotificationStateVersionConflict
 		}
 
 		cmd.ResultVersion = newVersion
@@ -453,9 +545,9 @@ func SetAlertNotificationStateToPendingCommand(ctx context.Context, cmd *m.SetAl
 	})
 }
 
-func GetOrCreateAlertNotificationState(ctx context.Context, cmd *m.GetOrCreateNotificationStateQuery) error {
+func GetOrCreateAlertNotificationState(ctx context.Context, cmd *models.GetOrCreateNotificationStateQuery) error {
 	return inTransactionCtx(ctx, func(sess *DBSession) error {
-		nj := &m.AlertNotificationState{}
+		nj := &models.AlertNotificationState{}
 
 		exist, err := getAlertNotificationState(sess, cmd, nj)
 
@@ -469,11 +561,11 @@ func GetOrCreateAlertNotificationState(ctx context.Context, cmd *m.GetOrCreateNo
 			return nil
 		}
 
-		notificationState := &m.AlertNotificationState{
+		notificationState := &models.AlertNotificationState{
 			OrgId:      cmd.OrgId,
 			AlertId:    cmd.AlertId,
 			NotifierId: cmd.NotifierId,
-			State:      m.AlertNotificationStateUnknown,
+			State:      models.AlertNotificationStateUnknown,
 			UpdatedAt:  timeNow().Unix(),
 		}
 
@@ -501,7 +593,7 @@ func GetOrCreateAlertNotificationState(ctx context.Context, cmd *m.GetOrCreateNo
 	})
 }
 
-func getAlertNotificationState(sess *DBSession, cmd *m.GetOrCreateNotificationStateQuery, nj *m.AlertNotificationState) (bool, error) {
+func getAlertNotificationState(sess *DBSession, cmd *models.GetOrCreateNotificationStateQuery, nj *models.AlertNotificationState) (bool, error) {
 	return sess.
 		Where("alert_notification_state.org_id = ?", cmd.OrgId).
 		Where("alert_notification_state.alert_id = ?", cmd.AlertId).
