@@ -1,14 +1,24 @@
-import React, { SFC } from 'react';
+import React, { FC } from 'react';
 import _ from 'lodash';
 import TopSectionItem from './TopSectionItem';
 import config from '../../config';
+import { getLocationSrv } from '@grafana/runtime';
 
-const TopSection: SFC<any> = () => {
+const TopSection: FC<any> = () => {
   const navTree = _.cloneDeep(config.bootData.navTree);
   const mainLinks = _.filter(navTree, item => !item.hideFromMenu);
+  const searchLink = {
+    text: 'Search',
+    icon: 'search',
+  };
+
+  const onOpenSearch = () => {
+    getLocationSrv().update({ query: { search: 'open' }, partial: true });
+  };
 
   return (
     <div className="sidemenu__top">
+      <TopSectionItem link={searchLink} onClick={onOpenSearch} />
       {mainLinks.map((link, index) => {
         return <TopSectionItem link={link} key={`${link.id}-${index}`} />;
       })}
