@@ -1,10 +1,12 @@
+import { GrafanaRootScope } from 'app/routes/GrafanaCtrl';
+
 export class Profiler {
   panelsRendered: number;
   enabled: boolean;
-  $rootScope: any;
+  $rootScope: GrafanaRootScope;
   window: any;
 
-  init(config, $rootScope) {
+  init(config: any, $rootScope: GrafanaRootScope) {
     this.$rootScope = $rootScope;
     this.window = window;
 
@@ -13,9 +15,9 @@ export class Profiler {
     }
   }
 
-  renderingCompleted(panelId) {
+  renderingCompleted() {
     // add render counter to root scope
-    // used by phantomjs render.js to know when panel has rendered
+    // used by image renderer to know when panel has rendered
     this.panelsRendered = (this.panelsRendered || 0) + 1;
 
     // this window variable is used by backend rendering tools to know
