@@ -1,21 +1,26 @@
 import _ from 'lodash';
-
-import kbn from 'app/core/utils/kbn';
-
 import { PanelCtrl } from 'app/features/panel/panel_ctrl';
-import { getExploreUrl } from 'app/core/utils/explore';
-import { applyPanelTimeOverrides, getResolution } from 'app/features/dashboard/utils/panel';
+import { applyPanelTimeOverrides } from 'app/features/dashboard/utils/panel';
 import { ContextSrv } from 'app/core/services/context_srv';
-import { toLegacyResponseData, TimeRange, LoadingState, DataFrame, toDataFrameDTO } from '@grafana/data';
-
-import { LegacyResponseData, DataSourceApi, PanelData, DataQueryResponse } from '@grafana/ui';
+import {
+  DataFrame,
+  DataQueryResponse,
+  DataSourceApi,
+  LegacyResponseData,
+  LoadingState,
+  PanelData,
+  PanelEvents,
+  TimeRange,
+  toDataFrameDTO,
+  toLegacyResponseData,
+} from '@grafana/data';
 import { Unsubscribable } from 'rxjs';
 import { PanelModel } from 'app/features/dashboard/state';
+import { PanelQueryRunner } from '../dashboard/state/PanelQueryRunner';
 
 class MetricsPanelCtrl extends PanelCtrl {
   scope: any;
   datasource: DataSourceApi;
-  $q: any;
   $timeout: any;
   contextSrv: ContextSrv;
   datasourceSrv: any;
@@ -28,13 +33,12 @@ class MetricsPanelCtrl extends PanelCtrl {
   timeInfo?: string;
   skipDataOnInit: boolean;
   dataList: LegacyResponseData[];
-  querySubscription?: Unsubscribable;
+  querySubscription?: Unsubscribable | null;
   useDataFrames = false;
 
   constructor($scope: any, $injector: any) {
     super($scope, $injector);
 
-    this.$q = $injector.get('$q');
     this.contextSrv = $injector.get('contextSrv');
     this.datasourceSrv = $injector.get('datasourceSrv');
     this.timeSrv = $injector.get('timeSrv');
@@ -42,8 +46,16 @@ class MetricsPanelCtrl extends PanelCtrl {
     this.scope = $scope;
     this.panel.datasource = this.panel.datasource || null;
 
-    this.events.on('refresh', this.onMetricsPanelRefresh.bind(this));
-    this.events.on('panel-teardown', this.onPanelTearDown.bind(this));
+    this.events.on(PanelEvents.refresh, this.onMetricsPanelRefresh.bind(this));
+    this.events.on(PanelEvents.panelTeardown, this.onPanelTearDown.bind(this));
+    this.events.on(PanelEvents.componentDidMount, this.onMetricsPanelMounted.bind(this));
+  }
+
+  private onMetricsPanelMounted() {
+    const queryRunner = this.panel.getQueryRunner() as PanelQueryRunner;
+    this.querySubscription = queryRunner
+      .getData({ withTransforms: true, withFieldConfig: true })
+      .subscribe(this.panelDataObserver);
   }
 
   private onPanelTearDown() {
@@ -71,7 +83,7 @@ class MetricsPanelCtrl extends PanelCtrl {
       // Defer panel rendering till the next digest cycle.
       // For some reason snapshot panels don't init at this time, so this helps to avoid rendering issues.
       return this.$timeout(() => {
-        this.events.emit('data-snapshot-load', data);
+        this.events.emit(PanelEvents.dataSnapshotLoad, data);
       });
     }
 
@@ -96,23 +108,23 @@ class MetricsPanelCtrl extends PanelCtrl {
       return;
     }
 
-    this.loading = false;
     this.error = err.message || 'Request Error';
-    this.inspector = { error: err };
 
     if (err.data) {
       if (err.data.message) {
         this.error = err.data.message;
-      }
-      if (err.data.error) {
+      } else if (err.data.error) {
         this.error = err.data.error;
       }
     }
 
-    console.log('Panel data error:', err);
-    return this.$timeout(() => {
-      this.events.emit('data-error', err);
-    });
+    this.angularDirtyCheck();
+  }
+
+  angularDirtyCheck() {
+    if (!this.$scope.$root.$$phase) {
+      this.$scope.$digest();
+    }
   }
 
   // Updates the response with information from the stream
@@ -121,12 +133,12 @@ class MetricsPanelCtrl extends PanelCtrl {
       if (data.state === LoadingState.Error) {
         this.loading = false;
         this.processDataError(data.error);
-        return;
       }
 
       // Ignore data in loading state
       if (data.state === LoadingState.Loading) {
         this.loading = true;
+        this.angularDirtyCheck();
         return;
       }
 
@@ -148,57 +160,37 @@ class MetricsPanelCtrl extends PanelCtrl {
         const legacy = data.series.map(v => toLegacyResponseData(v));
         this.handleQueryResult({ data: legacy });
       }
+
+      this.angularDirtyCheck();
     },
   };
 
   updateTimeRange(datasource?: DataSourceApi) {
     this.datasource = datasource || this.datasource;
     this.range = this.timeSrv.timeRange();
-    this.resolution = getResolution(this.panel);
 
     const newTimeData = applyPanelTimeOverrides(this.panel, this.range);
     this.timeInfo = newTimeData.timeInfo;
     this.range = newTimeData.timeRange;
 
-    this.calculateInterval();
-
     return this.datasource;
   }
 
-  calculateInterval() {
-    let intervalOverride = this.panel.interval;
-
-    // if no panel interval check datasource
-    if (intervalOverride) {
-      intervalOverride = this.templateSrv.replace(intervalOverride, this.panel.scopedVars);
-    } else if (this.datasource && this.datasource.interval) {
-      intervalOverride = this.datasource.interval;
-    }
-
-    const res = kbn.calculateInterval(this.range, this.resolution, intervalOverride);
-    this.interval = res.interval;
-    this.intervalMs = res.intervalMs;
-  }
-
   issueQueries(datasource: DataSourceApi) {
     this.datasource = datasource;
 
     const panel = this.panel as PanelModel;
     const queryRunner = panel.getQueryRunner();
 
-    if (!this.querySubscription) {
-      this.querySubscription = queryRunner.getData().subscribe(this.panelDataObserver);
-    }
-
     return queryRunner.run({
       datasource: panel.datasource,
       queries: panel.targets,
       panelId: panel.id,
       dashboardId: this.dashboard.id,
-      timezone: this.dashboard.timezone,
+      timezone: this.dashboard.getTimezone(),
+      timeInfo: this.timeInfo,
       timeRange: this.range,
-      widthPixels: this.resolution, // The pixel width
-      maxDataPoints: panel.maxDataPoints,
+      maxDataPoints: panel.maxDataPoints || this.width,
       minInterval: panel.interval,
       scopedVars: panel.scopedVars,
       cacheTimeout: panel.cacheTimeout,
@@ -214,7 +206,7 @@ class MetricsPanelCtrl extends PanelCtrl {
     }
 
     try {
-      this.events.emit('data-frames-received', data);
+      this.events.emit(PanelEvents.dataFramesReceived, data);
     } catch (err) {
       this.processDataError(err);
     }
@@ -233,24 +225,11 @@ class MetricsPanelCtrl extends PanelCtrl {
     }
 
     try {
-      this.events.emit('data-received', result.data);
+      this.events.emit(PanelEvents.dataReceived, result.data);
     } catch (err) {
       this.processDataError(err);
     }
   }
-
-  async getAdditionalMenuItems() {
-    const items = [];
-    if (this.contextSrv.hasAccessToExplore() && this.datasource) {
-      items.push({
-        text: 'Explore',
-        icon: 'gicon gicon-explore',
-        shortcut: 'x',
-        href: await getExploreUrl(this.panel, this.panel.targets, this.datasource, this.datasourceSrv, this.timeSrv),
-      });
-    }
-    return items;
-  }
 }
 
 export { MetricsPanelCtrl };
