@@ -1,15 +1,18 @@
 import { PanelModel } from 'app/features/dashboard/state/PanelModel';
 import {
+  DataLink,
+  DisplayValue,
   FieldDisplay,
-  LinkModelSupplier,
+  formattedValueToString,
+  getFieldDisplayValuesProxy,
   getTimeField,
   Labels,
-  ScopedVars,
+  LinkModelSupplier,
   ScopedVar,
-  Field,
-  LinkModel,
+  ScopedVars,
 } from '@grafana/data';
 import { getLinkSrv } from './link_srv';
+import { config } from 'app/core/config';
 
 interface SeriesVars {
   name?: string;
@@ -29,24 +32,33 @@ interface ValueVars {
   calc?: string;
 }
 
-interface DataLinkScopedVars extends ScopedVars {
+interface DataViewVars {
+  name?: string;
+  refId?: string;
+  fields?: Record<string, DisplayValue>;
+}
+
+interface DataLinkScopedVars {
   __series?: ScopedVar<SeriesVars>;
   __field?: ScopedVar<FieldVars>;
   __value?: ScopedVar<ValueVars>;
+  __data?: ScopedVar<DataViewVars>;
 }
 
 /**
  * Link suppliers creates link models based on a link origin
  */
-
 export const getFieldLinksSupplier = (value: FieldDisplay): LinkModelSupplier<FieldDisplay> | undefined => {
   const links = value.field.links;
   if (!links || links.length === 0) {
     return undefined;
   }
+
   return {
-    getLinks: (_scopedVars?: any) => {
-      const scopedVars: DataLinkScopedVars = {};
+    getLinks: (existingScopedVars?: any) => {
+      const scopedVars: DataLinkScopedVars = {
+        ...(existingScopedVars ?? {}),
+      };
 
       if (value.view) {
         const { dataFrame } = value.view;
@@ -60,8 +72,8 @@ export const getFieldLinksSupplier = (value: FieldDisplay): LinkModelSupplier<Fi
         };
 
         const field = value.colIndex !== undefined ? dataFrame.fields[value.colIndex] : undefined;
+
         if (field) {
-          console.log('Full Field Info:', field);
           scopedVars['__field'] = {
             value: {
               name: field.name,
@@ -69,26 +81,40 @@ export const getFieldLinksSupplier = (value: FieldDisplay): LinkModelSupplier<Fi
             },
             text: 'Field',
           };
-        }
 
-        if (value.rowIndex) {
-          const { timeField } = getTimeField(dataFrame);
-          scopedVars['__value'] = {
-            value: {
-              raw: field.values.get(value.rowIndex),
-              numeric: value.display.numeric,
-              text: value.display.text,
-              time: timeField ? timeField.values.get(value.rowIndex) : undefined,
-            },
-            text: 'Value',
-          };
+          if (value.rowIndex !== undefined && value.rowIndex >= 0) {
+            const { timeField } = getTimeField(dataFrame);
+            scopedVars['__value'] = {
+              value: {
+                raw: field.values.get(value.rowIndex),
+                numeric: value.display.numeric,
+                text: formattedValueToString(value.display),
+                time: timeField ? timeField.values.get(value.rowIndex) : undefined,
+              },
+              text: 'Value',
+            };
+          }
+
+          // Expose other values on the row
+          if (value.view) {
+            scopedVars['__data'] = {
+              value: {
+                name: dataFrame.name,
+                refId: dataFrame.refId,
+                fields: getFieldDisplayValuesProxy(dataFrame, value.rowIndex!, {
+                  theme: config.theme,
+                }),
+              },
+              text: 'Data',
+            };
+          }
         } else {
           // calculation
           scopedVars['__value'] = {
             value: {
               raw: value.display.numeric,
               numeric: value.display.numeric,
-              text: value.display.text,
+              text: formattedValueToString(value.display),
               calc: value.name,
             },
             text: 'Value',
@@ -98,14 +124,14 @@ export const getFieldLinksSupplier = (value: FieldDisplay): LinkModelSupplier<Fi
         console.log('VALUE', value);
       }
 
-      return links.map(link => {
-        return getLinkSrv().getDataLinkUIModel(link, scopedVars, value);
+      return links.map((link: DataLink) => {
+        return getLinkSrv().getDataLinkUIModel(link, scopedVars as ScopedVars, value);
       });
     },
   };
 };
 
-export const getPanelLinksSupplier = (value: PanelModel): LinkModelSupplier<PanelModel> => {
+export const getPanelLinksSupplier = (value: PanelModel): LinkModelSupplier<PanelModel> | undefined => {
   const links = value.links;
 
   if (!links || links.length === 0) {
@@ -120,17 +146,3 @@ export const getPanelLinksSupplier = (value: PanelModel): LinkModelSupplier<Pane
     },
   };
 };
-
-export const getLinksFromLogsField = (field: Field, rowIndex: number): Array<LinkModel<Field>> => {
-  const scopedVars: any = {};
-  scopedVars['__value'] = {
-    value: {
-      raw: field.values.get(rowIndex),
-    },
-    text: 'Raw value',
-  };
-
-  return field.config.links
-    ? field.config.links.map(link => getLinkSrv().getDataLinkUIModel(link, scopedVars, field))
-    : [];
-};
